"""
Josephus Problem: Double Step 
(Special Case of Step Size = 2)


"""


def doubling_paramstudy():

    print("%8s\t%8s\t%16s"%("n","# cycles","cycle lengths"))
    print("-"*60)

    #for n in range(4,100+1):
    #    mapp = factor_doubling_map_cycles(make_doubling_map(n))
    #    print("%8d\t%8d\t%s"%(n, len(mapp), ",".join([str(len(mm)) for mm in mapp])))

    for n in range(17):
        mapp = factor_doubling_map_cycles(make_doubling_map(2**n))
        print("%8d\t%8d\t%s"%(2**n, len(mapp), ",".join([str(len(mm)) for mm in mapp])))

def doubling_printnice(n):

    doublemap = make_doubling_map(n)
    
    if False:
        print_doubling_table()

    all_cycles = factor_doubling_map_cycles(doublemap)

    print("Factored doubling permutation for n = %d:"%(n))
    for cycle in all_cycles:
        s = " ".join([str(j) for j in cycle])
        print("( %s )"%(s))


def factor_doubling_map_cycles(doublemap):
    """Factor a doubling map 
    into a list of lists of integers
    representing its respective cycles
    """
    all_elements = set([j for j in doublemap.keys()])
    all_cycles = []

    # Progressively whittle down elements
    # until all are stored in cycles
    while len(all_elements) != 0:

        # Store the elements used in this cycle
        cycle = []

        # Initial condition is to start with 1,
        # first time through the loop ONLY.
        # Otherwise, initial condition is to
        # use the smallest element from
        # all_elements
        if len(all_cycles)==0:
            init = 1
        else:
            init = sorted(list(all_elements))[0]

        # Fencepost algorithm
        cycle.append(init)
        if init in all_elements:
            # should be removed, but just in case
            all_elements.remove(init)

        nxt = doublemap[init]

        # Continue until we detect a loop
        while nxt != init:
            cycle.append(nxt)
            all_elements.remove(nxt)
            ths = nxt
            nxt = doublemap[ths]

        all_cycles.append(cycle)

    return all_cycles


def make_doubling_map(n):
    """Make a map of doubling pairs:
    j maps to 2j mod (2n+1)
    """
    doublemap = {}
    for j in range(1,2*n+1):
        doublemap[j] = (2*j)%(2*n+1)
    return doublemap


def print_doubling_table():
    """Print the table/map of doubling pairs"""
    print("")
    print("n = %4d"%(n))
    print("")
    print("%8s\t%8s\t%8s"%("j","2j","2j mod 2n+1"))
    print("-"*60)
    for j in range(1,2*n+1):
        print("%8d\t%8d\t%8d"%(j,2*j,(2*j)%(2*n+1)))


if __name__=="__main__":
    #doubling_printnice(8)
    doubling_paramstudy()

